from math import sqrt
from checking_routines import smallest_unit
from basic_routines import map_coordinates

beta_threshold = 1.18  # ">=1.17683 !"
betaprecision = 10**7

def coordinate_to_int(c):
    "scale coordinate to integer"
    return int(c/smallest_unit) # smallest_unit=0.0001

def length2(p):
    """squared length of vector"""
    return p[0]**2 + p[1]**2

def checkbeta(piece):
  min_cos_squared = 100*betaprecision # infinity
  for part in piece["partlist"]:
    # scaling all coordinates does not affect beta-skeleton:
    poly = map_coordinates(coordinate_to_int, piece[part])
    for i in range(len(poly)-1): # run through all edges:
      p1 = poly[i]
      p2 = poly[i+1]
      # check edge (p1,p2):
      # run through all points p: 
      for part_2 in piece["partlist"]:
        for p in map_coordinates(coordinate_to_int,piece[part_2]):
          if p<>p1 and p<>p2:
            # compute cos_squared, representing the
            # squared cosine of the angle alpha=p1,p,p2
            x1 = (p1[0]-p[0], p1[1]-p[1])
            x2 = (p2[0]-p[0], p2[1]-p[1])
            innerproduct = x1[0]*x2[0] + x1[1]*x2[1]
            # so far, all calculations were exact.
            if innerproduct>0:
               cos_squared = ( innerproduct**2 * betaprecision /
                                      (length2(x1)*length2(x2)) )
               # scaled by betaprecision and rounded down;
               # yields a conservative (lower) estimate of beta
            min_cos_squared = min(min_cos_squared,cos_squared)
  min_cos_squared_unscaled = min_cos_squared/float(betaprecision)
  beta = 1/sqrt(1-min_cos_squared_unscaled)
  # This final calculation is just in double-precision floating-point.
  print "cos(alpha)^2 = %d/%d = %8.6f; beta = %8.6f." % (
    min_cos_squared, betaprecision, min_cos_squared_unscaled, beta)
  if beta < beta_threshold:
    print "beta too small: ", beta, ">=1.17683 !", beta_threshold
    raise RuntimeError
  return beta
