from decimal import Decimal,Context,Inexact
from pieces import thinwire
from basic_routines import translate_poly

#### CRUCIAL CONSTANTS #####
smallest_unit = Decimal("0.0001")
smallest_terminal_unit = Decimal("0.01")
from pieces import ratio_thick_thin # = Decimal("4.3")

#### CHECK: MAXIMUM NUMBER OF TRAILING DIGITS

precision_check_Context=Context(traps=[Inexact])

def checkprecision(piece):
    """raises an exception if some coordinate
       is not a multiple of smallest_unit"""
    for part in piece["partlist"]:
        for p in piece[part]:
	   precision_check_Context.quantize(p[0],smallest_unit)
	   precision_check_Context.quantize(p[1],smallest_unit)
    print "All coordinates are multiples of",smallest_unit

#### CHECK: PRESENCE OF W-LEFT OR W-RIGHT FOR ALL TERMINAL TRIANGLES

  ## ROTATIONS: ##
def rot90left(p):   return (-p[1],p[0])
def rot90right(p):  return (p[1],-p[0])
rotations =  (lambda x:x), rot90left, rot90right

def stretch(p,factor):
  "scale a point p by a factor"
  return (p[0]*factor,p[1]*factor)

def checkterminals(piece):
    t1base = piece["lower"][0]
    t1other = ("L",piece["upper"][-1]),("R",piece["upper"][-2])
    t2base = piece["lower"][-1]
    t2other = ("L",piece["upper"][1]),("R",piece["upper"][0])

    if len(piece["delta"])==2:
      terminals = (
         (t1base, "W-right", "lower","upper", t1other), 
         (t2base, "W-left",  "lower","upper", t2other), )
    else: # C-connection piece
      t2base = piece["right"][-1]
      t3base = piece["lower"][-1]
      t3other = ("l",piece["right"][1]),("r",piece["right"][0])
      terminals =(
         (t1base, "W-right", "lower","upper", t1other), 
         (t2base, "W-left",  "right","upper", t2other),
         (t3base, "W-left",  "lower","right", t3other), )

    for pbase,whichhalf,basepiece,otherpiece,two_points in terminals:
        tria = []
        print "terminal triangle basepoint: (%s,%s)"% pbase
        precision_check_Context.quantize(pbase[0],smallest_terminal_unit)
        precision_check_Context.quantize(pbase[1],smallest_terminal_unit)
        for letter,p_other in two_points:
            diffvector = ((p_other[0]-pbase[0]), (p_other[1]-pbase[1]))
            tria.append(diffvector)
            print ("     edge vector in state %s: (%s,%s)"%
                                 ((letter,)+diffvector))
        rotation, scalefactor = identify_triangle(tria)
        tu=[rotation(stretch(p,scalefactor))
                                     for p in thinwire["upper"]]
        tl=[rotation(stretch(p,scalefactor))
                                     for p in thinwire["lower"]]
        if whichhalf=="W-left":
            reference = tl[-1]
        else:
            reference = tl[0]
        tu=translate_poly(tu,reference, pbase)
        tl=translate_poly(tl,reference, pbase)

        for i in range(8): # check the 8 extreme points from lower
          if whichhalf=="W-right":
              j = i
          else:
              j = -1-i
          if tl[j]<>piece[basepiece][j]:
            print "End does not fit.",i,j,tl[j],piece[basepiece][j]
            raise RuntimeError
        for i in range(5): # check the 5 extreme points from upper
          if whichhalf=="W-left":
              j = i
          else:
              j = -1-i
          if tu[j]<>piece[otherpiece][j]:
            print "End does not fit.",i,j,tu[j],piece[otherpiece][j]
            raise RuntimeError
    print "All terminal coordinates are multiples of", \
                                      smallest_terminal_unit

def identify_triangle(tria):
    """determine which type of terminal triangle is at
    the piece (size and orientation)

    return values:
    rotation = identity or rotation by 90 degrees left or right
    scalefactor = 1 or 4.3
    """

    for scalefactor in (1, ratio_thick_thin):
      for rotation in rotations:
        if tria == [rotation(stretch(map(Decimal,p),scalefactor))
                for p in ('-2.7', '11.2'), ('2.7', '11.2')]:
            return rotation,scalefactor
    print "triangle", tria, "not recognized"
    raise RuntimeError


#### CHECK: DO THE PIECES HAVE ANY SYMMETRIES?

  ## SYMMETRIES: ##
def vertical_axis (p):          return (-p[0],p[1])
vertical_axis.axis =            "y-axis"

def vertical_axis_shifted (p):  return (Decimal("0.01")-p[0],p[1])
vertical_axis_shifted.axis =    "vertical axis x=0.005"

def plus_45_degree_axis(p):     return (p[1],p[0])
plus_45_degree_axis.axis =      "line x=y"

def minus_45_degree_axis(p):    return (-p[1],-p[0])
minus_45_degree_axis.axis =     "line x=-y"

symmetries = (vertical_axis, vertical_axis_shifted,
              plus_45_degree_axis, minus_45_degree_axis)

def checksymmetry(piece):
    lis = []
    for part in piece["partlist"]: lis.extend(piece[part])
    a = set(lis)
    print len(lis)-len(a),"duplicate point(s)."
    for sym in symmetries:
      # try out mapping under which the piece is symmetric
      if a == set(map(sym,a)):
	 print "The point set is symmetric with respect to the",\
                      sym.axis+"."
	 return
    print "The point set is not symmetric."
    if piece["symmetric"]: # piece SHOULD be symmetric
       raise RuntimeError
