from decimal import Decimal,getcontext
if getcontext().prec < 15:  # just to make sure.
     getcontext().prec = 15 # (Default precision should be sufficient.)

#### CRUCIAL CONSTANTS #####
delta_small = Decimal("5.655172")
delta_large = Decimal("24.0575")
ratio_thick_thin = Decimal("4.3")

"""Each piece has the following fields: (= keys)

"name", "filename": for displaying in tables or figures
"symmetric": True if non-symmetric piece should raise an error condition
"partlist": sequence of (names of) boundary parts.
   the default is ("lower","upper"); only C has three parts.
   When the parts are chained together in this order
   they form the counterclockwise boundary of the piece.
"lower","upper" (and possibly "right"): the actual boundary parts,
   as lists of points. Each point is a pair of "Decimal" numbers.
"delta": sequence of delta values associated to the (two or three)
   terminal triangles of the piece (delta_small or delta_large),
"patterns": an encoding of the terminal triangles (no longer used)
"labelpos", "vertshift", "scale": used for drawing and labeling the pieces
"""

from pieces_data import *
"""pieces_data contains the "raw" data from with the pieces are built,
in the form of lists of strings or string pairs"""

defaultpartlist = ("lower","upper")

#### PATTERNS (partially obsolete, used for drawing-routines) ####
"""
Patterns identify the terminal triangles.

The last item is the tip of the triangle (usually at
the bottom, as the "basepoint").
"""
branchpatterns = (
    ("upper",((0,"L"),(-1,"R")),"lower",0),
         # format: 0 = unchanged
         # 1= take one from front, -1 = from back
    ("upper",((1,"L"),(0,"R")), "right",-1),
    ("right",((1,"l"),(0,"r")), "lower",-1)
     )
tubepatterns = ( # for a tube with two ends
    ("upper",((0,"L"),(-1,"R")),"lower",0),
    ("upper",((1,"L"),(0,"R")),"lower",-1)
    )


from basic_routines import map_coordinates, translate, translate_poly

thinwire = { "name": "wire-piece", "filename":"wire",
             "symmetric": True,
             "labelpos": (20,0), "vertshift":25, "scale":12,
             "delta": (delta_small, delta_small),
             "partlist": defaultpartlist, "patterns": tubepatterns    }

# define coordinates for thin wire:
hori = Decimal('13.7') # horizontal half-period
vert = Decimal('0.1') # depth of intrusion
otherpoints = map_coordinates(Decimal, [('2.7','11.2'),('7.5','8.1'),('10.4','4.8')])
tri = otherpoints[0]

left = [(-hori-tri[0],tri[1])] + [(-hori+x[0],x[1]) for x in otherpoints]
right= [(-x[0],x[1]) for x in left]
left.reverse()
thinwire["upper"] = right + [(Decimal(0),-vert)] + left

left = [(-x[0],-vert-x[1]) for x in otherpoints]+[(-hori,Decimal(0))]
right= [(-x[0],x[1]) for x in left]
left.reverse()
thinwire["lower"] = left + right

phasewire = {"name": "extended wire-piece", "filename":"wire-extended",
             "symmetric": True,
             "labelpos": (45,0), "vertshift":25, "scale":12,
             "delta": (delta_small, delta_small),
             "partlist": defaultpartlist, "patterns": tubepatterns    }

# construct coordinates for extended (thin) wire:
llow= thinwire["lower"]
upp = thinwire["upper"]
leftanchor  = translate(llow[0], destination=(Decimal("-0.005"),0))
rightanchor = translate(llow[-1],destination=(Decimal("+0.005"),0))
leftlow = translate_poly(llow,llow[-1],leftanchor)
leftup  = translate_poly(upp, llow[-1],leftanchor)
rightlow = translate_poly(llow,llow[0],rightanchor)
rightup  = translate_poly(upp, llow[0],rightanchor)
up  = rightup[:-2] + upp  + leftup[2:]
low = leftlow + llow[1:-1] + rightlow
phasewire["lower"]=translate_poly(low,(0,0),(Decimal("+0.005"),0))
phasewire["upper"]=translate_poly(up ,(0,0),(Decimal("+0.005"),0))

resize = {"name": "thickening adapter", "filename":"resize",
          "symmetric": False,
          "labelpos": (-15,-50), "vertshift":110, "scale":50,
          "delta": (delta_small, delta_large),
          "partlist": defaultpartlist, "patterns": tubepatterns,
          "upper": map_coordinates(Decimal,resize_upper),
	  "lower": map_coordinates(Decimal,resize_lower)}

### set W for Lemma 5.1  ############################

pup = thinwire["upper"]
plow= thinwire["lower"]

# either edge [x,y] or edge [x,z] should
# belong to every optimal solution:
W_point_x = plow[-1]; W_point_y = pup[1]; W_point_z = pup[0];

pup2 =  translate_poly(pup[:-2],plow[0],plow[-1])
plow2 = translate_poly(plow[1:],plow[0],plow[-1])

W_upper = pup2[4:]+pup[:-4]
W_lower = plow+plow2


############################################

C = {"name": "$C$ connection", "filename": "C",
"symmetric": False,
"labelpos": (670,90), "vertshift":200,  "scale": 70,
"delta": (delta_large, delta_large, delta_small),
"partlist": ("lower","right","upper"), "patterns": branchpatterns,
 "upper": map_coordinates(Decimal,C_upper),
 "right": map_coordinates(Decimal,C_right),
 "lower": map_coordinates(Decimal,C_lower)
}


C0 = {"name": "$C_0$ connection", "filename": "C0",
"symmetric": True,
"labelpos": (670,90), "vertshift":170,  "scale": 70,
"delta": (delta_large, delta_large),
"partlist": defaultpartlist, "patterns": tubepatterns,
"upper": map_coordinates(Decimal,C0_upper),
"lower": map_coordinates(Decimal,C0_lower) }

############################################

thinrightangle = {"name": "right bend", "filename": "right-bend",
"symmetric": True,           
"labelpos": (544,-47),
"vertshift":80, "horishift": -33,  "scale": 30,
"delta": (delta_small, delta_small),
"partlist": defaultpartlist, "patterns": tubepatterns}

l=len(right_lower_x)
thinrightangle["lower"] = map_coordinates(Decimal,
    [(right_lower_x[i],right_lower_x[l-1-i]) for i in range(l)] )
l=len(right_upper_x)
thinrightangle["upper"] = map_coordinates(Decimal,
    [(right_upper_x[i],right_upper_x[l-1-i]) for i in range(l)] )

############################################


thinleftangle = {"name": "left bend", "filename": "left-bend",
"symmetric": True,           
"labelpos": (543,50),
"vertshift":70, "horishift": 33, "scale": 30,
"delta": (delta_small, delta_small),
"partlist": defaultpartlist, "patterns": tubepatterns,}

l=len(left_lower_x)
thinleftangle["lower"] = \
 [ (Decimal(left_lower_x[i]), -Decimal(left_lower_x[l-1-i])) for i in range(l)]
l=len(left_upper_x)
thinleftangle["upper"] = \
 [ (Decimal(left_upper_x[i]), -Decimal(left_upper_x[l-1-i])) for i in range(l)]

    
thickleftangle = {"name": "thick left bend", "filename": "thick-left-angle",
"symmetric": True,           
"labelpos": (-200,-2500),
"vertshift":210, "horishift": 120,  "scale":120,
"delta": (delta_large, delta_large),
"partlist": defaultpartlist, "patterns": tubepatterns}

thickleftangle["lower"]=map_coordinates(lambda x:x*ratio_thick_thin,
                          thinleftangle["lower"])
thickleftangle["upper"]=map_coordinates(lambda x:x*ratio_thick_thin,
                          thinleftangle["upper"])

allpieces = ( thinwire, phasewire, resize,
              C, C0, thinleftangle, thinrightangle, thickleftangle )
