import mwt_polygon
from basic_routines import map_coordinates
from arithmetic import decimal_to_scaled_int
import arithmetic

from output_routines import (write_table, show_solution,
                             startipefile, finishipefile)

compute_digits=15
displaydigits=9

def solve_all_patterns(piece,
       tablefile=None,picturefile=False,condensed_tablefile=None):
  """calculate costs for all combinations of terminal states
  for a piece with two or three terminal triangles.

  two or three nested loops, depending on length of piece["delta"]
  """

  arithmetic.setprecision(compute_digits)

  if picturefile:
     out=startipefile("results/TABLE-"+piece["filename"]+".ipe")
     s = 20.0 / piece["scale"]
     out.write('<group layer="alpha" matrix="%5.3f 0 0 %5.3f">\n'
               % (s,s))

  resultlist=[]

  ## set up data for nested loops:
  delta1 = decimal_to_scaled_int(piece["delta"][0])
  loop1 = ((len(piece["upper"]),  "L",-delta1),
           (len(piece["upper"])-1,"R",+delta1))

  delta2 = decimal_to_scaled_int(piece["delta"][1])
  loop2 = (1,"L",+delta2),(0,"R",-delta2)

  if len(piece["delta"])==3: # C-connection has THREE terminals
    delta3 = decimal_to_scaled_int(piece["delta"][2])
    loop3 = (1,"l",+delta3),(0,"r",-delta3)
  else:
    loop3 = ((None,"",0),) # third loop is "void"

  for endindex,       key1, d1 in loop1:
    for startindex,    key2, d2 in loop2:
      for startindex_r, key3, d3 in loop3:
        key=key1+key2+key3

        p = piece["upper"][startindex:endindex] + piece["lower"]
        if len(piece["delta"])==3: # C-connection with 3 terminals
          p += piece["right"][startindex_r:]
        print "case",key+":",len(p),"points.",
          
        pcalc = map_coordinates (decimal_to_scaled_int,p)
        elist = mwt_polygon.mwt(pcalc)
        for cost,sol,f in elist:
           if f: # a degenerate solution might be optimal
              print "solution",sol,"contains degenerate triangle."
              raise RuntimeError
        cost,sol,f = elist[0] # take the first solution
        reducedcost = cost + (d1+d2+d3)
        resultlist.append([key,len(elist),cost,reducedcost])
        print cost.quantize_and_print(displaydigits),
        print reducedcost.quantize_and_print(displaydigits)
        if picturefile:
            show_solution(out,piece,len(resultlist),p,sol,
                key+': '+ cost.quantize_and_print(displaydigits))
  if picturefile:
      out.write('</group>\n')
      finishipefile(out)

  # calculate relative reduced cost:
  minvalue = min(x[3].a for x in resultlist)
  for x in resultlist:
        x.append(x[3] + (-minvalue))

  write_table(tablefile, condensed_tablefile,
              piece["name"], resultlist, displaydigits)
    
